/*	Copyright: 	 Copyright 2002 Apple Computer, Inc. All rights reserved.

	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
			("Apple") in consideration of your agreement to the following terms, and your
			use, installation, modification or redistribution of this Apple software
			constitutes acceptance of these terms.  If you do not agree with these terms,
			please do not use, install, modify or redistribute this Apple software.

			In consideration of your agreement to abide by the following terms, and subject
			to these terms, Apple grants you a personal, non-exclusive license, under Apples
			copyrights in this original Apple software (the "Apple Software"), to use,
			reproduce, modify and redistribute the Apple Software, with or without
			modifications, in source and/or binary forms; provided that if you redistribute
			the Apple Software in its entirety and without modifications, you must retain
			this notice and the following text and disclaimers in all such redistributions of
			the Apple Software.  Neither the name, trademarks, service marks or logos of
			Apple Computer, Inc. may be used to endorse or promote products derived from the
			Apple Software without specific prior written permission from Apple.  Except as
			expressly stated in this notice, no other rights or licenses, express or implied,
			are granted by Apple herein, including but not limited to any patent rights that
			may be infringed by your derivative works or by other works in which the Apple
			Software may be incorporated.

			The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
			WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
			WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
			PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
			COMBINATION WITH YOUR PRODUCTS.

			IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
			OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
			(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
			ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*=============================================================================
	MusicDeviceBase.cpp
	
=============================================================================*/

#include "MusicDeviceBase.h"

MusicDeviceBase::MusicDeviceBase(		ComponentInstance				inInstance, 
										UInt32							numInputs,
										UInt32							numOutputs,
										UInt32							numGroups) 
	: AUBase(inInstance, numInputs, numOutputs, numGroups),
	  AUMIDIBase(this)
{
}

ComponentResult		MusicDeviceBase::GetPropertyInfo(AudioUnitPropertyID			inID,
												AudioUnitScope				inScope,
												AudioUnitElement			inElement,
												UInt32 &					outDataSize,
												Boolean &					outWritable)
{
	ComponentResult result;
	
	result = AUBase::GetPropertyInfo (inID, inScope, inElement, outDataSize, outWritable);
	
	if (result == kAudioUnitErr_InvalidProperty)
		result = AUMIDIBase::DelegateGetPropertyInfo (inID, inScope, inElement, outDataSize, outWritable);
	
	return result;
}

ComponentResult		MusicDeviceBase::GetProperty(	AudioUnitPropertyID 	inID,
												AudioUnitScope 				inScope,
												AudioUnitElement		 	inElement,
												void *						outData)
{
	ComponentResult result;
	
	result = AUBase::GetProperty (inID, inScope, inElement, outData);
	
	if (result == kAudioUnitErr_InvalidProperty)
		result = AUMIDIBase::DelegateGetProperty (inID, inScope, inElement, outData);
	
	return result;
}


#if PRAGMA_STRUCT_ALIGN
	#pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
	#pragma pack(2)
#endif
#if	TARGET_API_MAC_OS8 || TARGET_API_MAC_OSX
struct MusicDevicePrepareInstrumentGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	MusicDeviceInstrumentID        inInstrument;
	MusicDeviceComponent           ci;
};
struct MusicDeviceReleaseInstrumentGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	MusicDeviceInstrumentID        inInstrument;
	MusicDeviceComponent           ci;
};
struct MusicDeviceStartNoteGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	const MusicDeviceNoteParams*   inParams;
	UInt32                         inOffsetSampleFrame;
	NoteInstanceID*                outNoteInstanceID;
	MusicDeviceGroupID             inGroupID;
	MusicDeviceInstrumentID        inInstrument;
	MusicDeviceComponent           ci;
};
struct MusicDeviceStopNoteGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	UInt32                         inOffsetSampleFrame;
	NoteInstanceID                 inNoteInstanceID;
	MusicDeviceGroupID             inGroupID;
	MusicDeviceComponent           ci;
};
#elif	TARGET_OS_WIN32
struct MusicDevicePrepareInstrumentGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	long                           inInstrument;
};
struct MusicDeviceReleaseInstrumentGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	long                           inInstrument;
};
struct MusicDeviceStartNoteGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	long                           inInstrument;
	long                           inGroupID;
	long                           outNoteInstanceID;
	long                           inOffsetSampleFrame;
	long                           inParams;
};
struct MusicDeviceStopNoteGluePB {
	unsigned char                  componentFlags;
	unsigned char                  componentParamSize;
	short                          componentWhat;
	long                           inGroupID;
	long                           inNoteInstanceID;
	long                           inOffsetSampleFrame;
};
#else
	#error	Platform not supported
#endif
#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
	#pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
	#pragma pack()
#endif


ComponentResult		MusicDeviceBase::ComponentEntryDispatch(	ComponentParameters *			params,
																MusicDeviceBase *				This)
{
	if (This == NULL) return paramErr;

	ComponentResult result;
	
	switch (params->what) {
	case kMusicDeviceMIDIEventSelect:
	case kMusicDeviceSysExSelect:
		{
			result = AUMIDIBase::ComponentEntryDispatch (params, This);
		}
		break;
	case kMusicDevicePrepareInstrumentSelect:
		{
			MusicDevicePrepareInstrumentGluePB *pb = (MusicDevicePrepareInstrumentGluePB *)params;
			#if	!TARGET_OS_WIN32
				MusicDeviceInstrumentID	pbinInstrument = pb->inInstrument;
			#else
				MusicDeviceInstrumentID	pbinInstrument = (*((MusicDeviceInstrumentID*)&pb->inInstrument));
			#endif
			
			result = This->PrepareInstrument(pbinInstrument);
		}
		break;
	case kMusicDeviceReleaseInstrumentSelect:
		{
			MusicDeviceReleaseInstrumentGluePB *pb = (MusicDeviceReleaseInstrumentGluePB *)params;
			#if	!TARGET_OS_WIN32
				MusicDeviceInstrumentID	pbinInstrument = pb->inInstrument;
			#else
				MusicDeviceInstrumentID	pbinInstrument = (*((MusicDeviceInstrumentID*)&pb->inInstrument));
			#endif
			
			result = This->ReleaseInstrument(pbinInstrument);
		}
		break;
	case kMusicDeviceStartNoteSelect:
		{
			MusicDeviceStartNoteGluePB *pb = (MusicDeviceStartNoteGluePB *)params;
			#if	!TARGET_OS_WIN32
				MusicDeviceInstrumentID			pbinInstrument = pb->inInstrument;
				MusicDeviceGroupID				pbinGroupID = pb->inGroupID;
				NoteInstanceID*					pboutNoteInstanceID = pb->outNoteInstanceID;
				UInt32							pbinOffsetSampleFrame = pb->inOffsetSampleFrame;
				const MusicDeviceNoteParams*	pbinParams = pb->inParams;
			#else
				MusicDeviceInstrumentID			pbinInstrument = (*((MusicDeviceInstrumentID*)&pb->inInstrument));
				MusicDeviceGroupID				pbinGroupID = (*((MusicDeviceGroupID*)&pb->inGroupID));
				NoteInstanceID*					pboutNoteInstanceID = (*((NoteInstanceID**)&pb->outNoteInstanceID));
				UInt32							pbinOffsetSampleFrame = (*((UInt32*)&pb->inOffsetSampleFrame));
				const MusicDeviceNoteParams*	pbinParams = (*((const MusicDeviceNoteParams**)&pb->inParams));
			#endif
			
			result = This->StartNote(pbinInstrument, pbinGroupID, pboutNoteInstanceID, pbinOffsetSampleFrame, pbinParams);
		}
		break;
	case kMusicDeviceStopNoteSelect:
		{
			MusicDeviceStopNoteGluePB *pb = (MusicDeviceStopNoteGluePB *)params;
			#if	!TARGET_OS_WIN32
				MusicDeviceGroupID				pbinGroupID = pb->inGroupID;
				NoteInstanceID					pbinNoteInstanceID = pb->inNoteInstanceID;
				UInt32							pbinOffsetSampleFrame = pb->inOffsetSampleFrame;
			#else
				MusicDeviceGroupID				pbinGroupID = (*((MusicDeviceGroupID*)&pb->inGroupID));
				NoteInstanceID					pbinNoteInstanceID = (*((NoteInstanceID*)&pb->inNoteInstanceID));
				UInt32							pbinOffsetSampleFrame = (*((UInt32*)&pb->inOffsetSampleFrame));
			#endif
			
			result = This->StopNote(pbinGroupID, pbinNoteInstanceID, pbinOffsetSampleFrame);
		}
		break;

	default:
		result = AUBase::ComponentEntryDispatch(params, This);
		break;
	}
	
	return result;
}

