#include <math.h>
#include "Voweler.h"
// -----------------------------------------------------------------------------
// Component entry point and registration
// -----------------------------------------------------------------------------
// The next two lines define the entry point for the component and the
// IDs the component uses to identify itself to the system. The name of the
// component entry _must_ be the name of the class you derive from
// MusicDeviceBase and _must_ match the component entry string in the resource
// file (in case of this example the string "CVoweler" in the Voweler.r file).
// The same is true for the first parameter of the register statement.
// The second parameter of COMPONENT_REGISTER must alway be kAudioUnitType_MusicDevice for
// the component type is a music device. The thrid parameter identifies your
// effect, the fourth one should be your manufacturer ID you got from Apple.
COMPONENT_ENTRY(CVoweler);
#define kFirstAUParameter 0
// -----------------------------------------------------------------------------
// CVoweler
// -----------------------------------------------------------------------------
// Coeff for Vowel filter.
// These coeffs are from alex@smartelectronix.com, found on MusicDsp.org
#define kNumberOfVowels 6
static const double kCoeff[][11]= {
{ 8.11044e-06,
    8.943665402,	-36.83889529,	92.01697887,	-154.337906,	181.6233289,
    -151.8651235,  	89.09614114,	-35.10298511,	8.388101016 ,	-0.923313471
},
{4.36215e-06,
    8.90438318,		-36.55179099,	91.05750846,	-152.422234,	179.1170248,
    -149.6496211,	87.78352223,	-34.60687431,	8.282228154,	-0.914150747
},
{ 3.33819e-06,
    8.893102966,	-36.49532826,	90.96543286,	-152.4545478,	179.4835618,
    -150.315433,	88.43409371,	-34.98612086,	8.407803364,	-0.932568035
},
{1.13572e-06,
    8.994734087,	-37.2084849,	93.22900521,	-156.6929844,	184.596544,
    -154.3755513,	90.49663749,	-35.58964535,	8.478996281,	-0.929252233
},
{4.09431e-07,
    8.997322763,	-37.20218544,	93.11385476,	-156.2530937,	183.7080141,
    -153.2631681,	89.59539726,	-35.12454591,	8.338655623,	-0.910251753
},
{1, // No Filter
    0,	0,	0,	0,	0,
    0,	0,	0,	0,	0
}
};
// -----------------------------------------------------------------------------
enum {
    kVowelFilter1 = kFirstAUParameter,
    kVowelFilter2,
    kVowelMix
};

#if 0
#define NAME(X) X, CFSTR(X)
#else
#define NAME(X) X, 0, 0, CFSTR(X)
#endif

#define kParamReadable kAudioUnitParameterFlag_IsReadable
#define kParamWritable kAudioUnitParameterFlag_IsWritable
#define kCFString kAudioUnitParameterFlag_HasCFNameString
#define kParamGeneric kAudioUnitParameterUnit_Generic
#define kParamIndexed kAudioUnitParameterUnit_Indexed
#define kParamBPM kAudioUnitParameterUnit_BPM
#define kParamPercent kAudioUnitParameterUnit_Percent

const AudioUnitParameterInfo CVoweler::s_asParInfo[]= {
    // Name         	 CFStr Unit                       Min   Max   Default Flags
  {  NAME("AEIOU.(1)"), kParamIndexed, 1, kNumberOfVowels, kNumberOfVowels,   kParamReadable | kParamWritable | kCFString    },
  {  NAME("AEIOU.(2)"), kParamIndexed, 1, kNumberOfVowels, kNumberOfVowels,   kParamReadable | kParamWritable | kCFString    },
  {  NAME("Mix"), kParamPercent, 0.f, 100.f, 50.f,   kParamReadable | kParamWritable | kCFString    }
};

const UInt32 CVoweler::s_nbPar=sizeof(CVoweler::s_asParInfo)/sizeof(AudioUnitParameterInfo);
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	CVoweler::CVoweler
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CVoweler::CVoweler(	AudioUnit		audioUnit) :
AUEffectBase(audioUnit)
{
    for (int i = 0; i < s_nbPar; ++i) {
        SetParameter(kFirstAUParameter+i, s_asParInfo[i].defaultValue);
    }
}
// -----------------------------------------------------------------------------
ComponentResult CVoweler::Initialize(void)
{
    UInt32 in = GetInput(0)->GetStreamFormat().NumberChannels();
    UInt32 out = GetOutput(0)->GetStreamFormat().NumberChannels();
    if ((in != out) || (in < 1) || (in > 2)) {
        return kAudioUnitErr_FormatNotSupported;
    }
    return AUEffectBase::Initialize();
}
// -----------------------------------------------------------------------------
ComponentResult CVoweler::GetParameterInfo(AudioUnitScope iScope, AudioUnitParameterID iParID, AudioUnitParameterInfo& rsInfo)
{
    // Returns info on a single parameter.
    if (iScope==kAudioUnitScope_Global && iParID >= kFirstAUParameter)
    {
        memcpy(&rsInfo, &s_asParInfo[iParID-kFirstAUParameter], sizeof(AudioUnitParameterInfo));
        return noErr;
    }
    return AUEffectBase::GetParameterInfo(iScope, iParID, rsInfo);
}
// -----------------------------------------------------------------------------
int			CVoweler::GetNumCustomUIComponents ()
{
    return 1;
}
// -----------------------------------------------------------------------------
void			CVoweler::GetUIComponentDescs (ComponentDescription* inDescArray)
{
    inDescArray->componentType=kAudioUnitCarbonViewComponentType;
    inDescArray->componentSubType='Vowl';
    inDescArray->componentManufacturer='Airy';
    inDescArray->componentFlags=0;
    inDescArray->componentFlagsMask=0;
}
// -----------------------------------------------------------------------------
UInt32			CVoweler::SupportedNumChannels (const AUChannelInfo** outInfo)
{
    static AUChannelInfo info[] = {{1, 1},{2, 2}};
    if (outInfo) {
        *outInfo = info;
    }
    return sizeof(info)/sizeof(*info);
}
// -----------------------------------------------------------------------------
ComponentResult CVoweler::ChangeStreamFormat(AudioUnitScope iScope, AudioUnitElement iElem, const CAStreamBasicDescription& sOld, const CAStreamBasicDescription& sNew)
{
    const UInt32 iCha=sNew.NumberChannels();
    if (sOld.NumberChannels()!=sNew.NumberChannels())
    {
        const UInt32 iCha=sNew.NumberChannels();
        if ((iCha<1) || (iCha> 2) || (sNew.NumberInterleavedChannels()>1)) {
            return kAudioUnitErr_FormatNotSupported;
        }
    }
    return AUEffectBase::ChangeStreamFormat(iScope, iElem, sOld, sNew);
}
// -----------------------------------------------------------------------------
ComponentResult CVoweler::GetParameterValueStrings(AudioUnitScope iScope, AudioUnitParameterID iID, CFArrayRef* tArrayRef)
{
    if (iScope==kAudioUnitScope_Global)
    {
        if (iID == kVowelFilter1 || iID == kVowelFilter2) {
            // Hand the menu strings to the host.
            static const void* ppVal[]={ CFSTR("A"), CFSTR("E"), CFSTR("I"), CFSTR("O"), CFSTR("U"), CFSTR("none") };
            if (tArrayRef) *tArrayRef=CFArrayCreate(NULL, ppVal, kNumberOfVowels, NULL);
            return noErr;
        }
    }
    return AUBase::GetParameterValueStrings(iScope, iID, tArrayRef);
}
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
float CVoweler::CVowelerKernel::Filter(float val, int type1, int type2, float mix)
{
    double res = val;
    if (type1 <= kNumberOfVowels && type2 <= kNumberOfVowels) {
        double res1, res2;
        if (type1 < kNumberOfVowels) {
            register const double *coeff = kCoeff[type1-1];
            res1 = coeff[0]*val +
                coeff[1] * mMemory1[0] +
                coeff[2] * mMemory1[1] +
                coeff[3] * mMemory1[2] +
                coeff[4] * mMemory1[3] +
                coeff[5] * mMemory1[4] +
                coeff[6] * mMemory1[5] +
                coeff[7] * mMemory1[6] +
                coeff[8] * mMemory1[7] +
                coeff[9] * mMemory1[8] +
                coeff[10] * mMemory1[9];
            mMemory1[9]= mMemory1[8];
            mMemory1[8]= mMemory1[7];
            mMemory1[7]= mMemory1[6];
            mMemory1[6]= mMemory1[5];
            mMemory1[5]= mMemory1[4];
            mMemory1[4]= mMemory1[3];
            mMemory1[3]= mMemory1[2];
            mMemory1[2]= mMemory1[1];
            mMemory1[1]= mMemory1[0];
            mMemory1[0]=(double) res1;
        } else {
            res1 = val;
        }
        if (type2 < kNumberOfVowels) {
            register const double *coeff = kCoeff[type2-1];
            res2 = coeff[0]*val +
                coeff[1] * mMemory2[0] +
                coeff[2] * mMemory2[1] +
                coeff[3] * mMemory2[2] +
                coeff[4] * mMemory2[3] +
                coeff[5] * mMemory2[4] +
                coeff[6] * mMemory2[5] +
                coeff[7] * mMemory2[6] +
                coeff[8] * mMemory2[7] +
                coeff[9] * mMemory2[8] +
                coeff[10] * mMemory2[9];
            mMemory2[9]= mMemory2[8];
            mMemory2[8]= mMemory2[7];
            mMemory2[7]= mMemory2[6];
            mMemory2[6]= mMemory2[5];
            mMemory2[5]= mMemory2[4];
            mMemory2[4]= mMemory2[3];
            mMemory2[3]= mMemory2[2];
            mMemory2[2]= mMemory2[1];
            mMemory2[1]= mMemory2[0];
            mMemory2[0]=(double) res2;
        } else {
            res2 = val;
        }
        res = res1*(1.-mix)+res2*mix;
    }
    return res;
}
// -----------------------------------------------------------------------------
void 	CVoweler::CVowelerKernel::Process(	const Float32 * 	inSourceP,
                                         Float32 * 			inDestP,
                                         UInt32 				inFramesToProcess,
                                         UInt32				inNumChannels,
                                         bool &				ioSilence )
{
    int type1 = GetParameter(kVowelFilter1);
    int type2 = GetParameter(kVowelFilter2);
    float mix = GetParameter(kVowelMix)/100.;
    for (UInt32 i=0; i<inFramesToProcess; ++i) {
        // write output
        *inDestP = Filter(*inSourceP, type1, type2, mix);
        inDestP += inNumChannels;
        inSourceP += inNumChannels;
    }
    ioSilence = false;
}
// -----------------------------------------------------------------------------
// End of file.
// -----------------------------------------------------------------------------
