#ifndef __VOWELER_H__
#define __VOWELER_H__

// Effect plugins are based on this baseclass
#include "AUEffectBase.h"
#include "Voweler_constantes.h"

// -----------------------------------------------------------------------------
// Class CVoweler
//
// This class implements a simple vowel filter plugin
//
// -----------------------------------------------------------------------------

class CVoweler: public AUEffectBase
{
public:

                CVoweler(	AudioUnit		audioUnit);
	
		// Baseclass overrides -----------------------
		ComponentResult		// error code
		GetParameterInfo	// Get info on a single parameter.
		(AudioUnitScope,	// scope
		 AudioUnitParameterID,		// parameter ID
		 AudioUnitParameterInfo&);	// return for parameter info

                ComponentResult	Version()
                {
                    return VOWELER_VERSION;
                }

                ComponentResult			Initialize ();
		int			GetNumCustomUIComponents ();
		void			GetUIComponentDescs (ComponentDescription* inDescArray);
		UInt32			SupportedNumChannels (	const AUChannelInfo**			outInfo);
		 ComponentResult GetParameterValueStrings(AudioUnitScope iScope, AudioUnitParameterID iID,
					   CFArrayRef* tArrayRef);

		virtual AUKernelBase* NewKernel() { return new CVowelerKernel(this); }

		ComponentResult ChangeStreamFormat(AudioUnitScope iScope, AudioUnitElement iElem, const CAStreamBasicDescription& sOld, const CAStreamBasicDescription& sNew);
		
		class CVowelerKernel : public AUKernelBase		// most real work happens here
		{
public:
		    CVowelerKernel(AUEffectBase *inAudioUnit )
		    : AUKernelBase(inAudioUnit)
		{
			for (int i = 0; i < 10; i++)
			    mMemory1[i] = mMemory2[i] = 0;
		};

		    // processes one channel of interleaved samples
		    virtual void 		Process( const Float32 * inSourceP,
			      Float32 * inDestP,
			      UInt32 	inFramesToProcess,
			      UInt32	inNumChannels,
			      bool &	ioSilence);

		    virtual void Reset()
		    {
			for (int i = 0; i < 10; i++)
			    mMemory1[i] = mMemory2[i] = 0;
		    }

private:
		    virtual	float Filter(float val, int type1, int type2, float mix);

		    float mMemory1[10];
		    float mMemory2[10];
		};
private:
		// Statics -----------------------------------
		static const UInt32			s_nbPar;	// number of parameters
		static const AudioUnitParameterInfo s_asParInfo[];	// parameter info array

};

#endif
// -----------------------------------------------------------------------------
// End of file.
// -----------------------------------------------------------------------------
