/*
 *  AUCarbonViewNib.cpp
 *  AUNib
 *
 *  Created by Airy Andr on Sun Oct 06 2002.
 *  Copyright (c) 2002-2003 Airy Andr. All rights reserved.
 *
 */

#include "AUCarbonViewNib.h"
#include "AUGUIUtilities.h"
#include "AUCarbonViewEnhancedControl.h"
#include "TPngPictControl.h"
#include "TValuePictControl.h"
#include "TValuePictButtonControl.h"
#include "TKnobControl.h"
#include "TSliderControl.h"
#include "TMeterControl.h"
#include "TMeterPartialControl.h"
#include "TValueTextControl.h"
#include "TImageButton.h"
#include "TImagePopup.h"
#include "TTransparentEditText.h"
#include "TMultiPane.h"

#include "TImageCache.h"

#define kMaxNumberOfControlsForEachParameter 20
#define kMaxNumberOfMultipanesForEachPane 20

//	The timer proc
extern "C" static void TimerProc ( EventLoopTimerRef inTimer, void *inUserData );

static void TimerProc ( EventLoopTimerRef inTimer, void *inUserData )
{
	if ( inUserData != NULL ) {
		CAUCarbonViewNib *view = (CAUCarbonViewNib *)inUserData;
		view->Idle();
	}
}

// -----------------------------------------------------------------------------
// Class CAUCarbonViewNib
//
// !This class generates a custom view of the plugin from a nib window.
// -----------------------------------------------------------------------------

static pascal void BackgroundEraseRectProc( GrafVerb verb, const Rect* r )
{
    if( verb != kQDGrafVerbErase ) {
		// not interested
		StdRect( verb, r );
    } else {
		CAUCarbonViewNib *view = 0;

		StdRect( verb, r );
		CGrafPtr currPort;
		GetPort( &currPort);
		WindowRef theWin = GetWindowFromPort(currPort);
		if (theWin) {
			GetWindowProperty(theWin, 'AGUI', 'CVNB', sizeof(CAUCarbonViewNib *), 0, &view);
		}
		if (view && view->GetBackgroundImage()) {
			CGContextRef context;
			Rect rect;

			QDBeginCGContext(currPort, &context);
			GetPortBounds(currPort, &rect);
			CGContextTranslateCTM(context, 0, float(rect.bottom-rect.top));
			CGContextScaleCTM(context, 1, -1);

			TRect trect = *r;
			CGContextClipToRect(context, trect);

			Rect paneRect;
			view->GetPaneBounds(&paneRect);
			trect = paneRect;
			HIViewDrawCGImage(context, &trect, view->GetBackgroundImage());

			QDEndCGContext (currPort, &context );
		}
    }
}
// -----------------------------------------------------------------------------
CAUCarbonViewNib::CAUCarbonViewNib(AudioUnitCarbonView tView):
AUCarbonViewBase(tView), mCurrentProcsPtr(0), mBackgroundImage(0), mBundleRef(0), mRootUserPane(0), mPaneID(0)
{
}
// -----------------------------------------------------------------------------
CAUCarbonViewNib::~CAUCarbonViewNib()
{
	if (HasTimer())
		RemoveEventLoopTimer(mTimer);

    if (mBackgroundImage) {
		SetPortGrafProcs( GetWindowPort( mCarbonWindow ), mCurrentProcsPtr);
		DisposeRoutineDescriptor (mProcs.rectProc);
		CGImageRelease(mBackgroundImage);
    }
    // Delete our userPane so we can unregister our TView classes
    ControlID controlID = { 'AUid', kAUNibUserPaneControlID }; // My user pane
    ControlRef control;
    if (GetControlByID (mCarbonWindow, &controlID, &control) == noErr) {
		if (HasBackgroundPict()) {
			// My user pane is embededed in the background pane
			HIViewRef superview = HIViewGetSuperview(control);
			if (superview)
				DisposeControl(superview);
			else
				DisposeControl(control);
		} else {
			DisposeControl(control);
		}
    }
    // UnRegister standard SDK controls
    // Note : this fails with MacOS X < 10.2.4 (bug in MacOS X)
   TViewNoCompositingCompatible::UnRegisterAllClasses();
    TImageCache::Reset();

	// Deactivate all the fonts from our bundle
	CFURLRef myResourcesURL = CFBundleCopyResourcesDirectoryURL(mBundleRef);
	if (myResourcesURL != NULL)
	{
		FSRef myResourceDirRef;
		FSSpec bundleResourceDirFSSpec;
		CFURLGetFSRef(myResourcesURL, &myResourceDirRef);
		OSStatus status = FSGetCatalogInfo(&myResourceDirRef, kFSCatInfoNone, NULL, NULL, &bundleResourceDirFSSpec, NULL);
		if (status == noErr)
			FMDeactivateFonts(&bundleResourceDirFSSpec, NULL, NULL, kFMDefaultOptions);
		CFRelease( myResourcesURL );
	}
	
    CFRelease(mBundleRef);
}
// -----------------------------------------------------------------------------
//! Called before the GUI has been created
void CAUCarbonViewNib::InitWindow(CFBundleRef sBundle) // Register custom controls...
{
    HIViewRef		view;
	// From Apple's Code :
 // Workaround: Unfortunately, at the time we call RegisterClass below,
 // the HIView base class isn't registered. It's supposed to be automatically
 // registered, but something is going wrong in HIToolbox. We can force
 // it to register by creating any arbitrary view. Here, we simply create
 // and release a scroll view. That's enough to make sure the HIView base
 // class is registered. Sorry folks.

	// From me (AA) : Is it still needed ???
    HIScrollViewCreate( kHIScrollViewOptionsVertScroll, &view );
    CFRelease( view );
}
// -----------------------------------------------------------------------------
//! Called after the GUI has been created
void CAUCarbonViewNib::FinishWindow(CFBundleRef sBundle)
{
}
// -----------------------------------------------------------------------------
//! Create the AU GUI
OSStatus CAUCarbonViewNib::CreateUI(Float32 x, Float32 y)
{
    Rect controlFrame;
    WindowRef win;
    IBNibRef      inNibRef;

    mBundleID = GetBundleID();
    mBundleRef = CFBundleGetBundleWithIdentifier (GetBundleID());
    CFRetain (mBundleRef);
    InitWindow(mBundleRef);

    CreateNibReferenceWithCFBundle(mBundleRef, GetWindowNibName(), &inNibRef);

	// Activate all the fonts from our bundle
	CFURLRef myResourcesURL = CFBundleCopyResourcesDirectoryURL(mBundleRef);
	if (myResourcesURL != NULL)
	{
		FSRef myResourceDirRef;
		FSSpec bundleResourceDirFSSpec;
		CFURLGetFSRef(myResourcesURL, &myResourceDirRef);
		OSStatus status = FSGetCatalogInfo(&myResourceDirRef, kFSCatInfoNone, NULL, NULL, &bundleResourceDirFSSpec, NULL);
		if (status == noErr)
			status = FMActivateFonts(&bundleResourceDirFSSpec, NULL, NULL, kFMLocalActivationContext);
		CFRelease( myResourcesURL );
	}	

	HideControl(mCarbonPane);


    // Register standard SDK controls
TViewNoCompositingCompatible::SetClassBundle(mBundleRef);
TViewNoCompositingCompatible::RegisterClassForBundleID<TValuePictControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TPngPictControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TValuePictButtonControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TKnobControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TSliderControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TMeterControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TMeterPartialControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TValueTextControl>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TImageButton>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TImagePopup>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TTransparentEditText>(mBundleID);
TViewNoCompositingCompatible::RegisterClassForBundleID<TMultiPane>(mBundleID);

	WindowAttributes attributes;
    GetWindowAttributes(mCarbonWindow, &attributes);
    bool hasCompositing = attributes & kWindowCompositingAttribute;
	if (HasBackgroundPict() && !hasCompositing) { // Patching QD is only needed with non compositing windows
		mBackgroundImage = TImageCache::GetImage(mBundleRef, GetBackgroundFilename(), NULL, NULL);
		if (mBackgroundImage) {
			if (PatchEraseRect()) {
				// Add a property to the Window so anybody with a window ptr can get the AUCarbonViewNib
				CAUCarbonViewNib *ptr = this;
				SetWindowProperty(mCarbonWindow, 'AGUI', 'CVNB', sizeof(CAUCarbonViewNib *), &ptr);

				mCurrentProcsPtr = GetPortGrafProcs( GetWindowPort( mCarbonWindow ) );
				if ( mCurrentProcsPtr != NULL )
					mProcs = *mCurrentProcsPtr;
				else
					SetStdCProcs( &mProcs );
				mProcs.rectProc = NewQDRectUPP( BackgroundEraseRectProc );
				SetPortGrafProcs( GetWindowPort( mCarbonWindow ), &mProcs );
			}
		}
    }

TViewNoCompositingCompatible::SetWindowReady(false);
    if (!hasCompositing || (CreateWindowFromNib( inNibRef, GetWindowCompositingName(), &win) != noErr)) {
		CreateWindowFromNib( inNibRef, GetWindowNoCompositingName(), &win);
    }
    OSStatus result = noErr;
    ControlRef control;
#if 1
    ControlID controlID = { kAUNibControlSignature, kAUNibUserPaneControlID }; // The user pane to embed
    result = GetControlByID (win, &controlID, &control);
#else
    result = GetRootControl(win, &control); // Why this does not work ???
#endif
    if (result == noErr) {
		GetControlBounds (control, &controlFrame);
		OffsetRect (&controlFrame, (short)x, (short)y);
		SetControlBounds (control, &controlFrame);

		if (HasBackgroundPict()) {
			HIViewRef backCtrl;
			CFStringRef reg = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@.%@"), mBundleID,
											  TPngPictControl::GetControlName());
                        TViewNoCompositingCompatible::Create<TPngPictControl>(&backCtrl, 0, 0, reg);
			CFRelease(reg);
			SetControlTitleWithCFString(backCtrl, GetBackgroundFilename()) ;
			SetControlBounds (backCtrl, &controlFrame);
			EmbedControl( backCtrl );
			if (hasCompositing) {
				::HIViewAddSubview( backCtrl, control );
			} else {
				::EmbedControl( control, backCtrl );
			}
			ShowControl( backCtrl);
		} else {
			EmbedControl( control);
		}
    }
    TViewNoCompositingCompatible::SetWindowReady(true);
	mRootUserPane = control;

	// register for clicks in our root pane since we completely cover up
	// the actual root pane, mCarbonPane
	// We also register for kEventCommandProcess, to handle plug-in level commands and
        // kEventWindowContextualMenuSelect to handle MIDI learn on ctrl-click on controls
        EventTypeSpec events[] = { {kEventClassCommand, kEventCommandProcess} };
        WantEventTypes(GetControlEventTarget(mCarbonPane), GetEventTypeCount(events), events);
        EventTypeSpec events2[] = { {kEventClassWindow, kEventWindowContextualMenuSelect} };
        WantEventTypes(GetWindowEventTarget(mCarbonWindow), GetEventTypeCount(events2), events2);

	// now automatically build all the multipane (mpan, x) in the root user pane
	BuildDynamicViews(mRootUserPane);

	// now automatically bind all the controls in the root user pane
	BindPane(mRootUserPane);

        FinishWindow(mBundleRef);
	ReleaseWindow(win);
        DisposeNibReference(inNibRef);

	if (HasTimer())		
		InstallEventLoopTimer(
						GetCurrentEventLoop(),
						0,
						kEventDurationMillisecond * TimerResolution(),
						NewEventLoopTimerUPP( TimerProc ),
						this,
						&mTimer );
		
	ShowControl(mCarbonPane);
    return noErr;
}

// -----------------------------------------------------------------------------
void CAUCarbonViewNib::GetPaneBounds(Rect *r)
{
    GetControlBounds (mCarbonPane, r);
}

// -----------------------------------------------------------------------------
//! Test if the carbon window has the compositing attribute
bool CAUCarbonViewNib::WindowHasCompositing()
{
	WindowAttributes attributes;
	GetWindowAttributes(mCarbonWindow, &attributes);
	return attributes & kWindowCompositingAttribute;
}

// -----------------------------------------------------------------------------
//! Bind all the controls embedded the pane to their associated parameters
void CAUCarbonViewNib::BindPane(HIViewRef thePane)
{
	CGImageRef textBackground = 0;
	if (GetTextBackgroundFilename())
         textBackground = TImageCache::GetImage(mBundleRef, GetTextBackgroundFilename(), NULL, NULL);

	// Built the Font Styte for text controls
	OSStatus styleOK = 0;
	ControlFontStyleRec textStyle = { 0 };
	if (GetFontname()) {
		char fname[255];
		CFStringGetCString(GetFontname(), fname+1, sizeof(fname)-1, kCFStringEncodingASCII);
		*fname = strlen(fname+1);
		textStyle.flags = kControlUseFontMask | kControlUseSizeMask | kControlUseForeColorMask | kControlUseJustMask;
		textStyle.font = FMGetFontFamilyFromName((const unsigned char *)fname);
		textStyle.size = GetFontsize();
		textStyle.just = teJustLeft; // Should we have a GetSomething for that ?
		textStyle.foreColor.red = textStyle.foreColor.green = textStyle.foreColor.blue = (unsigned short)0; // Same here...
		styleOK = 1;
	}

	UInt32 size;
    Boolean writable;
    AudioUnitParameterID *ids;
    AudioUnitGetPropertyInfo(mEditAudioUnit,
							 kAudioUnitProperty_ParameterList,
							 kAudioUnitScope_Global,
							 0,
							 &size,
							 &writable);
    ids = new AudioUnitParameterID[size/sizeof(AudioUnitParameterID)];
    AudioUnitGetProperty(mEditAudioUnit,
						 kAudioUnitProperty_ParameterList,
						 kAudioUnitScope_Global,
						 0,
						 ids,
						 &size);

    for (unsigned int i = 0; i <  size/sizeof(AudioUnitParameterID); ++i) {
		CAAUParameter tParam(mEditAudioUnit, ids[i], kAudioUnitScope_Global, 0);
		ControlRef control;
                AUCarbonViewControl::ControlType controlType = AUCarbonViewControl::kTypeContinuous;
		for (int j = -1; j < kMaxNumberOfControlsForEachParameter; ++j) {
			HIViewID hiID = { kAUNibControlSignature, j*1000+ids[i] };
			if (j == -1)
				hiID.id = -1;
			if (HIViewFindByID(thePane, hiID, &control) == noErr) {
				int nbDigit = 5;
				
				// Bind control to parameter i
                            AudioUnitParameterID id;
                            if (j == -1 || GetControlProperty(control, 'AGUI', 'auid', sizeof(id), NULL, &id) != noErr) {
				ControlKind kind;
				GetControlKind(control, &kind);

				switch (kind.kind) {
					case kControlKindCheckBox:
					case kControlKindRadioGroup: {
						controlType = AUCarbonViewControl::kTypeDiscrete;
						break;
					}
					case kControlKindEditText:
					case kControlKindStaticText:
					case kControlKindEditUnicodeText: {
						controlType = AUCarbonViewControl::kTypeText;
						if (kind.signature == 'airy') {
							SetControlData(control,0, kTransparentEditTextBackgroundImageTag, sizeof(CGImageRef), &textBackground);
						}
						if (tParam.IsIndexedParam())
							nbDigit = (int)log10(tParam.ParamInfo().maxValue)+1;
						break;
					}
					case TMultiPane::kControlKind: {
						int mini, maxi, val;
						mini = int(tParam.ParamInfo().minValue);
						maxi = int(tParam.ParamInfo().maxValue);
						val = int(tParam.ParamInfo().defaultValue);
						SetControl32BitMinimum(control, mini);
						SetControl32BitMaximum(control, maxi);
						SetControl32BitValue(control, val);
						BuildMultiPane(control);
						break;
					}
					case kControlKindPopupArrow:
					case kControlKindPopupButton:
						if (tParam.HasNamedParams()) {
							MenuRef menuRef;
							CreateNewMenu( 1, 0, &menuRef);
							for (int i = 0; i < tParam.GetNumIndexedParams(); ++i)
							{
								AppendMenuItemTextWithCFString (menuRef, tParam.GetParamName(i), 0, 0, 0);
							}

							SetControlData(control, 0, kControlPopupButtonMenuRefTag, sizeof(menuRef), &menuRef);
							SetControl32BitMaximum(control, tParam.GetNumIndexedParams());
							controlType = AUCarbonViewControl::kTypeDiscrete;
						}
						break;
					default: {
						int mini, maxi;

						if (tParam.IsIndexedParam()) {
							controlType = AUCarbonViewControl::kTypeDiscrete;
							mini = int(tParam.ParamInfo().minValue);
							maxi = int(tParam.ParamInfo().maxValue);
						} else {
							controlType = AUCarbonViewControl::kTypeContinuous;
							mini = 0;
							maxi = 73513440;
						}
						SetControl32BitMinimum(control, mini);
						SetControl32BitMaximum(control, maxi);
						if (kind.kind == kControlKindSlider)
							SetControlAction(control, AUCarbonViewControl::SliderTrackProc);                                                
						break;
					}
				}
				if (styleOK)
					SetControlData(control,0, kControlFontStyleTag, sizeof(textStyle), &textStyle);
				AUCarbonViewControl *auvc = new AUCarbonViewEnhancedControl(this, mParameterListener,
																controlType, tParam, control, 0, 0, nbDigit);
//				auvc->Update(true); // To be sure the control has the right value
				auvc->Bind();
				AddControl(auvc);

				// Set a property so we can find the ParamID from the control
				SetControlProperty(control, 'AGUI', 'auid', sizeof(ids[i]), &ids[i]);
                            }
                                
			} else if (j >= 0) {
				// No more control for this parameter
				break;
			}
		}
	}
        delete[] ids;
	if (textBackground)
		CFRelease(textBackground);
#ifdef ATSU
	if (styleOK == noErr)
		ATSUDisposeStyle( textStyle );
#endif
}
// -----------------------------------------------------------------------------
//! Build all multipanes (kAUPanelControlSignature, x) contained in the pane
void CAUCarbonViewNib::BuildDynamicViews(HIViewRef thePane)
{
	for (int i = 0; i < kMaxNumberOfMultipanesForEachPane; ++i) {
		for (int j = 0; j < kMaxNumberOfMultipanesForEachPane; ++j) {
			HIViewID hiID = { kAUPanelControlSignature, i+j*1000 };
			ControlRef control;
			if (HIViewFindByID(thePane, hiID, &control) == noErr) {
				BuildMultiPane(control);
			} else {
				break;
			}
		}
	}
}

//! Populate the multipane "control", according to its title and its min,max
void CAUCarbonViewNib::BuildMultiPane(HIViewRef control)
{
	OSErr result;
	CFStringRef panelName;
        int id;
        GetControlData(control,0, TMultiPane::kSubPanelID, sizeof(id), &id, 0);
        if (id > -1) {
            // Hey ! I've already built this one
            return;
        }
        SetControlData(control,0, TMultiPane::kSubPanelID, sizeof(mPaneID), &mPaneID);
        mPaneID++;

        int paneID;
        GetControlData(control,0, TMultiPane::kSubPanelID, sizeof(paneID), &paneID, 0);
        
	CopyControlTitleAsCFString(control, &panelName);

	int mini = GetControl32BitMinimum(control);
	int maxi = GetControl32BitMaximum(control);
	int val = GetControl32BitValue(control);

	// Here, we can have three levels of backgrounds :
	// - the one for the AU window
	// - one for the multipane (<title>.png)
	// - one for the subpane (<title><value>.png)
	// All all these can have some transparency, making possibilities quite nice...
	HIViewRef backCtrl;
	CFStringRef backImageName = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@.png"), panelName);
	CGImageRef image = TImageCache::GetImage(mBundleRef, backImageName, NULL, NULL);
	bool imageFound = image != NULL;
	if (imageFound) {
		// Create a new default background
		CFStringRef reg = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@.%@"), mBundleID,
											 TPngPictControl::GetControlName());
                TViewNoCompositingCompatible::Create<TPngPictControl>(&backCtrl, 0, 0, reg);
		CFRelease(reg);
		Rect controlFrame;
		GetControlBounds (control, &controlFrame);
		SetControlBounds (backCtrl, &controlFrame);
		SetControlTitleWithCFString(backCtrl, backImageName) ;
		if (!WindowHasCompositing()) {
			result = ::EmbedControl( backCtrl, control);
		} else {
			TRect r = controlFrame;
			r.SetOrigin(0.f, 0.f);
			controlFrame = r;
			SetControlBounds (backCtrl, &controlFrame);
			result = ::HIViewAddSubview( control, backCtrl);
		}
		CFRelease(image);

		// Set its signature (mpan, 9999)
		ControlID controlID = {TMultiPane::kSubPanelSignature+paneID, 9999};
		SetControlID(backCtrl, &controlID);
	} else {
		backCtrl = control;
	}
	CFRelease(backImageName);

	IBNibRef      inNibRef;
	result = CreateNibReferenceWithCFBundle(mBundleRef, panelName, &inNibRef);
	for (int i = mini; i <= maxi; ++i) {
		ControlRef pane;
		WindowRef win;

		// Search for a local background for this pane
		HIViewRef back;
		CFStringRef backImageName = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@%d.png"), panelName, i);
		CGImageRef image = TImageCache::GetImage(mBundleRef, backImageName, NULL, NULL);
		bool imageFound = image != NULL;
		if (imageFound) {
			// Create a new default background
			CFStringRef reg = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@.%@"), mBundleID,
											  TPngPictControl::GetControlName());
                        TViewNoCompositingCompatible::Create<TPngPictControl>(&back, 0, 0, reg);
			CFRelease(reg);
			Rect controlFrame;
			GetControlBounds (backCtrl, &controlFrame);
			SetControlBounds (back, &controlFrame);
			SetControlTitleWithCFString(back, backImageName) ;
			if (!WindowHasCompositing()) {
				result = ::EmbedControl( back, backCtrl);
			} else {
				TRect r = controlFrame;
				r.SetOrigin(0.f, 0.f);
				controlFrame = r;
				SetControlBounds (back, &controlFrame);
				result = ::HIViewAddSubview( backCtrl, back);
			}
			CFRelease(image);
		} else {
			back = backCtrl;
		}
		CFRelease(backImageName);

		// Get the window
		CFStringRef winName = 0;
		OSStatus res = noErr;
		if (WindowHasCompositing()) {
			winName = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%dCompositing"), i);
			res = CreateWindowFromNib( inNibRef, winName, &win);
			CFRelease(winName);
		}
		if (res != noErr || !WindowHasCompositing()) {
			winName = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%d"), i);
			res = CreateWindowFromNib( inNibRef, winName, &win);
			CFRelease(winName);
		}

		if (res == noErr) {
			// Get its root control
			ControlID controlID = { kAUNibControlSignature, kAUNibUserPaneControlID }; // The user pane to embed
			result = GetControlByID (win, &controlID, &pane);
			if (result == noErr) {
				// Embed it into control
				if (WindowHasCompositing()) {
					result = ::HIViewAddSubview( back, pane );
				} else {
					Rect controlFrame;
					GetControlBounds (back, &controlFrame);
					SetControlBounds (pane, &controlFrame);
					result = ::EmbedControl( pane, back);
				}

				if (imageFound) {
					pane = back;
				}
				HIViewSetVisible( pane, i == val);
				// Set its signature (pane,i)
				controlID.signature = TMultiPane::kSubPanelSignature+paneID;
				controlID.id = i;
				result = SetControlID(pane, &controlID);

				// Bind it
				BindPane(pane);

				// Build multipanes contained in the pane.
				BuildDynamicViews(pane);
			}
			// Release the window
			ReleaseWindow(win);
		}
		// That's all...
	}
	ShowControl(backCtrl);

	CFRelease(panelName);
	DisposeNibReference(inNibRef);
}

//! We handle any clicks in the root user pane just like they are handled
//! in the AUCarbonViewBase class, namely by resetting keyboard focus. This
//! is done because our user pane completely covers up the actual root pane,
//! mCarbonPane, which AUCarbonViewBase is watching for clicks on.
bool CAUCarbonViewNib::HandleEvent(EventRef event)
{
	bool res = false;

	UInt32 eclass = GetEventClass(event);
	UInt32 ekind = GetEventKind(event);
        
        if ( (eclass == kEventClassWindow) && (ekind == kEventWindowContextualMenuSelect) ) 
        {
            Boolean res = false;
            ControlRef theControl = 0;
            HIPoint thePoint;
            Point testPoint;
            GetEventParameter( event, kEventParamMouseLocation, typeHIPoint, NULL, sizeof( thePoint ), NULL, &thePoint );
            testPoint.h = int(thePoint.x);
            testPoint.v = int(thePoint.y);
            AUGUI::ConvertGlobalToWindowPoint(mCarbonWindow, testPoint);
            FindControl(testPoint, mCarbonWindow, &theControl);
            if (theControl) {
                HandleControlContextualMenuClick(theControl, testPoint, &res);
                int id;
                if (GetControlProperty(theControl, 'AGUI', 'auid', sizeof(id), NULL, &id) == noErr && id != -1) {
                    AudioUnitSetProperty(mEditAudioUnit,
                                         kProperty_MIDILearn,
                                         kAudioUnitScope_Global,
                                         0,
                                         &id,
                                         sizeof(id));
                    res = true;
                }
            }
            return res;
        }
	// catch any clicks on the embedding pane in order to clear focus
	if ( (eclass == kEventClassControl) && (ekind == kEventControlClick) )
	{
		ControlRef clickedControl;
		GetEventParameter(event, kEventParamDirectObject, typeControlRef, NULL, sizeof(ControlRef), NULL, &clickedControl);
		if (clickedControl == mRootUserPane)
		{
			// reset focus
			ClearKeyboardFocus(GetControlOwner(mCarbonPane));
			res = true;
		}
	}
	if (!res && (eclass == kEventClassCommand) && (ekind == kEventCommandProcess))
	{
		HICommandExtended cmd;
		GetEventParameter( event, kEventParamDirectObject, typeHICommand, NULL, sizeof( cmd ), NULL, &cmd );
		switch (cmd.commandID) {
			case kAUPanelControlSignature:
				if (cmd.attributes & kHICommandFromControl)
				{
					HIViewRef source = cmd.source.control;
					// Get the control value
					SInt32 value = GetControl32BitValue(source);
					// Get the control ID
					HIViewID id;
					GetControlID(source, &id);
					res = SwitchPane(id.id % 1000, value);
				}
				break;
			case 'Abou': { // show "About" pict
				// Create a new 'PngPict' with the "about" title, and associate the "Rmve" command to it
				HIViewRef about;
				CGImageRef image = TImageCache::GetImage(mBundleRef, GetAboutFilename(), NULL, NULL);
				if (image != NULL) {

					CFStringRef reg = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%@.%@"), mBundleID,
												TPngPictControl::GetControlName());
                                        TViewNoCompositingCompatible::Create<TPngPictControl>(&about, 0, 0, reg);
					CFRelease(reg);
					Rect controlFrame;
					GetControlBounds (mRootUserPane, &controlFrame);
					SetControlBounds (about, &controlFrame);
					SetControlTitleWithCFString(about, GetAboutFilename()) ;
					if (!WindowHasCompositing()) {
						::EmbedControl( about, mCarbonPane);
						if (mCarbonPane != HIViewGetSuperview(mRootUserPane))
							HideControl(HIViewGetSuperview(mRootUserPane));
					} else {
						TRect r = controlFrame;
						r.SetOrigin(0.f, 0.f);
						controlFrame = r;
						SetControlBounds (about, &controlFrame);
						::HIViewAddSubview( mCarbonPane, about);
					}
					CFRelease(image);
					SetControlCommandID(about, 'Rmve');
					HIViewSetZOrder(about, kHIViewZOrderAbove, NULL);
					ShowControl(about);
					res = true;
				}
				break;
			}
			case 'Rmve': // remove "About" pict
				if (cmd.attributes & kHICommandFromControl)
				{
					HIViewRef ctrl = cmd.source.control;
					DisposeControl(ctrl);

					if (mCarbonPane != HIViewGetSuperview(mRootUserPane))
						ShowControl(HIViewGetSuperview(mRootUserPane));
					res = true;
				}
				break;
			default:
				res = HandleCommand(event, cmd);
				break;
		}
	}
	return res || AUCarbonViewBase::HandleEvent(event);
}

// switch pane (kAUPanelControlSignature,<paneID>) to paneNumber subpanel
int CAUCarbonViewNib::SwitchPane(int paneID, int paneNumber)
{
    HIViewID id = { kAUPanelControlSignature, paneID };
    HIViewRef pane;
	OSStatus result = HIViewFindByID(mRootUserPane, id, &pane);
	if (result == noErr) {
		SetControl32BitValue(pane, paneNumber);
	}
	return true;
}
// -----------------------------------------------------------------------------
// End of file.
// -----------------------------------------------------------------------------
