/*
 *  TXYControl.cpp
 *  CAUGuiDemo
 *
 *  Created by Airy on Sat May 24 2003.
 *  Copyright (c) 2003 Airy ANDRE. All rights reserved.
 *
 */

#include "TXYControl.h"
#include "TImageCache.h"
#include "AUGUIUtilities.h"

using namespace AUGUI;

// -----------------------------------------------------------------------------
//	macros
// -----------------------------------------------------------------------------
//

// -----------------------------------------------------------------------------
//	TXYControl constructor
// -----------------------------------------------------------------------------
//
TXYControl::TXYControl(
						   HIViewRef			inControl )
:	TViewNoCompositingCompatible( inControl ), mBackImage(0), mMarker(0),
mX(0), mY(0)
{
}

// -----------------------------------------------------------------------------
//	TXYControl destructor
// -----------------------------------------------------------------------------
//	Clean up after yourself.
//
TXYControl::~TXYControl()
{
    if (mBackImage)
		CGImageRelease(mBackImage);
    if (mMarker)
		CGImageRelease(mMarker);
}

// -----------------------------------------------------------------------------
//	GetKind
// -----------------------------------------------------------------------------
//
ControlKind TXYControl::GetKind()
{
    const ControlKind kMyKind = { 'airy', 'xypn' };
    return kMyKind;
}


//-----------------------------------------------------------------------------------
//	Initialize
//-----------------------------------------------------------------------------------
//	The control is set up.  Do the last minute stuff that needs to be done like
//	installing EventLoopTimers.
//
OSStatus TXYControl::Initialize(
								  TCarbonEvent&		inEvent )
{
#pragma unused( inEvent )
    TitleChanged();
    return noErr;
}
//-----------------------------------------------------------------------------------
//	ValueChanged
//-----------------------------------------------------------------------------------
//
void TXYControl::ValueChanged()
{
    Invalidate();
}
//-----------------------------------------------------------------------------------
//	TitleChanged
//	The title format is
//	 	filename
//	With :
//		<filename>.pgn and <filename>Back.png are the pict used by the control
//-----------------------------------------------------------------------------------
void TXYControl::TitleChanged()
{
    if (mMarker)
		CGImageRelease(mMarker);
    if (mBackImage)
		CGImageRelease(mBackImage);
    if ( mBundleRef )
    {
		CFStringRef fileName;
		CopyControlTitleAsCFString(GetViewRef(), &fileName);
		if ( fileName != NULL )
		{
			char name[100];
			char file[100];
			
			CFStringGetCString(fileName, file, 100, kCFStringEncodingASCII);
			
			sprintf(name, "%s.png", file);
			CFStringRef knob = CFStringCreateWithCStringNoCopy(0, name, kCFStringEncodingASCII, 0);
			mMarker = TImageCache::GetImage( mBundleRef, knob, NULL, NULL );

			sprintf(name, "%sBack.png", file);
			CFStringRef back = CFStringCreateWithCStringNoCopy(0, name, kCFStringEncodingASCII, 0);
			mBackImage = TImageCache::GetImage( mBundleRef, back, NULL, NULL );

			CFRelease( fileName );
		}
    }
}

//-----------------------------------------------------------------------------------
//	DrawStaticBackground
//-----------------------------------------------------------------------------------
//!	Draw the static part (background) of the control
//
void TXYControl::DrawStaticBackground(
										RgnHandle				inLimitRgn,
										CGContextRef			inContext,
										bool  inCompositing)
{
#pragma unused( inLimitRgn )
    TRect					bounds = Bounds();
    CGContextRef			context = inContext;

    if (mBackImage)
		HIViewDrawCGImage( context, &bounds, mBackImage);
}

//-----------------------------------------------------------------------------------
//	CompatibleDraw
//-----------------------------------------------------------------------------------
//!	The fun part of the control
//
void TXYControl::CompatibleDraw(
								  RgnHandle				inLimitRgn,
								  CGContextRef			inContext,
								  bool  inCompositing)
{
#pragma unused( inLimitRgn )
    if (mMarker) {
		TRect	bounds = Bounds();
		TRect	markerBounds = RectForCGImage(mMarker);
			
		markerBounds.MoveTo(
					  bounds.MinX() + mX*(bounds.MaxX() - bounds.MinX())-markerBounds.Width()/2,
					  bounds.MaxY() - mY*(bounds.MaxY() - bounds.MinY())-markerBounds.Height()/2);
		HIViewDrawCGImage( inContext, &markerBounds, mMarker);
    }
}

// -----------------------------------------------------------------------------
//	StartTracking
// -----------------------------------------------------------------------------
//!	Called before tracking.
//
OSStatus TXYControl::StartTracking(TCarbonEvent& inEvent, HIPoint& from)
{
    TRect bounds = Bounds();
	mX = from.x / bounds.Width();
	mY = 1. - from.y / bounds.Height();
	SetValue(!GetValue());
    return noErr;}

// -----------------------------------------------------------------------------
//	StillTracking
// -----------------------------------------------------------------------------
//!	Called during tracking.
//
OSStatus TXYControl::StillTracking(TCarbonEvent&inEvent, HIPoint& from)
{
    TRect bounds = Bounds();
	HIPoint mouse;
    inEvent.GetParameter<HIPoint>( kEventParamWindowMouseLocation, typeHIPoint, &mouse );
    ConvertToLocal(mouse);
	
    UInt32 modifiers;
    inEvent.GetParameter( kEventParamKeyModifiers, &modifiers );
	if ((modifiers == controlKey || modifiers == optionKey))
	{
		// value depends on how much the mouse moved
		double ratio = 1.;
		if (modifiers & controlKey)
			ratio = 50.;
		else if (modifiers & optionKey)
			ratio = 10.;
		mX += (mouse.x-from.x) / (bounds.Width() * ratio);
		mY -= (mouse.y-from.y) / (bounds.Height() * ratio);
	}
	else
	{
		// value is directly proportional to position
		mX = mouse.x / bounds.Width();
		mY = 1. - mouse.y / bounds.Height();
	}
	if (mX < 0.) mX = 0.;
	if (mX > 1.) mX = 1.;
	if (mY < 0.) mY = 0.;
	if (mY > 1.) mY = 1.;
	SetValue(!GetValue());
	from = mouse;
    return noErr;
}

OSStatus TXYControl::SetData(OSType inTag, ControlPartCode inPart, Size inSize, const void* inPtr)
{
	OSStatus err = noErr;
	switch (inTag)
	{
		case selectorX:
			if (inSize == sizeof(mX))
			{
				mX = *(double *)inPtr;
			}
			else
			{
				err = errDataSizeMismatch;
			}
			if (IsCompositing())
				Invalidate();
			else
				if (IsVisible()) Draw(0,0);
			break;
		case selectorY:
			if (inSize == sizeof(mY))
			{
				mY = *(double *)inPtr;
			}
			else
			{
				err = errDataSizeMismatch;
			}
			if (IsCompositing())
				Invalidate();
			else
				if (IsVisible()) Draw(0,0);
			break;
		default:
			// unsupported data tag
			err = errDataNotSupported;
	}
	return err;
}

OSStatus TXYControl::GetData(OSType inTag, ControlPartCode inPart, Size inSize, Size* outSize, void* inPtr)
{
	OSStatus err = noErr;
	switch (inTag)
	{
		case selectorX:
			if (inSize == sizeof(mX))
				*(double*)inPtr = mX;
			else
				err = errDataSizeMismatch;
			break;
		case selectorY:
			if (inSize == sizeof(mY))
				*(double*)inPtr = mY;
			else
				err = errDataSizeMismatch;
			break;

		default:
			err = TViewNoCompositingCompatible::GetData(inTag, inPart, inSize, outSize, inPtr);
	}
	return err;
}

