/*
 *  AUGUIDemoView.cpp
 *  CAUGuiDemo
 *
 *  Created by Airy on Wed Mar 19 2003.
 *  Copyright (c) 2003 Airy ANDRE. All rights reserved.
 *
 */

#include "AUGUIDemoView.h"
#include "CAUGuiDemoAU.h"
#include "TXYControl.h"
#include "AUCarbonViewEnhancedControl.h"

void CAUGuiDemoAUView::FinishWindow(CFBundleRef sBundle)
{
	// Find our meters
	for (int i = 0; i < kNumInternalParameters; ++i) {
		HIViewID id = { 'idle', i };
		OSStatus result = HIViewFindByID(mRootUserPane, id, &mMeter[i]);
		if (result == noErr) {
			mMeterMin[i] = GetControl32BitMinimum(mMeter[i]);
			mMeterMax[i] = GetControl32BitMaximum(mMeter[i]);
		} else {
			mMeter[i] = 0;
		}
	}

	// Find and register our XY panel as a controler for kParam_Knob_1 and kParam_Knob_2
	HIViewID id = { 'xypn', 0 };
	HIViewRef ctrl;
	OSStatus result = HIViewFindByID(mRootUserPane, id, &ctrl);
	if (result == noErr) {
		CAAUParameter tParamX(mEditAudioUnit, kParam_Knob_1, kAudioUnitScope_Global, 0);
		CAAUParameter tParamY(mEditAudioUnit, kParam_Knob_2, kAudioUnitScope_Global, 0);

		AUCarbonViewControl *auvc;
		auvc = new AUCarbonViewEnhancedControl(this, mParameterListener,
								 AUCarbonViewControl::kTypeContinuous, tParamX, ctrl, kControlNoPart, TXYControl::selectorX);
		auvc->Bind();
		auvc->Update(true); // To be sure the control has the right value
		AddControl(auvc);
		auvc = new AUCarbonViewEnhancedControl(this, mParameterListener,
                                                                AUCarbonViewControl::kTypeContinuous, tParamY, ctrl, kControlNoPart, TXYControl::selectorY);
		auvc->Bind();
		auvc->Update(true); // To be sure the control has the right value
		AddControl(auvc);
	}
        printf("Finish Window\n");
}

// -----------------------------------------------------------------------------
//! Called before the GUI has been created
void CAUGuiDemoAUView::InitWindow(CFBundleRef sBundle) // Register custom controls
{
TViewNoCompositingCompatible::RegisterClassForBundleID<TXYControl>(mBundleID);
}

// -----------------------------------------------------------------------------
// ! Idle code : called by the timer
void CAUGuiDemoAUView::Idle()
{
	if (mInternalParameters) {
		// Update our meters
		for (int i = 0; i < kNumInternalParameters; ++i) {
			if (mMeter[i]) {
				SetControl32BitValue(mMeter[i], SInt32(mMeterMin[i] + mInternalParameters[i]*(mMeterMax[i]-mMeterMin[i])));
			}
		}
	} else {
		UInt32 numParamsPassed = sizeof( float* );
		AudioUnitGetProperty( GetEditAudioUnit(), kInternalParameterProperyID,
						kAudioUnitScope_Global, 1, 	&mInternalParameters, &numParamsPassed );
	}
}