/*	Copyright: 	 Copyright 2002 Apple Computer, Inc. All rights reserved.

*/
/*=============================================================================
CAUGuiDemoAU.cpp

=============================================================================*/
#include <AudioToolbox/AudioUnitUtilities.h>
#include "CAUGuiDemoAU.h"

// ____________________________________________________________________________
//
COMPONENT_ENTRY(CAUGuiDemoAU)


// ____________________________________________________________________________
//
CAUGuiDemoAU::CAUGuiDemoAU(AudioUnit component)
: AUMIDIEffectBase(component)
{
    
    static const AUChannelInfo asChannelInfo[]= {
        // Inputs Outputs
    { 1, 1 },
    { 2, 2 },
    { 0, 0 }
    };
    
    m_asChannelInfo=asChannelInfo;
    m_arPar = NULL;
    
    // Internal parameters being read out during timed idle() calls
    
    for ( int i = 0; i < kNumInternalParameters; i++ )
    {
        internalParameters[ i ] = 0.f;			
    }
    
    // Make Presets
    
    static const EZ_preset asEZ_preset[]=
    {
        // Name			Floats
    {	"Preset 1",	{ 10.f, 100.f, 1.f, 0.f, 90.f, 100.f, 0.f, 0.f }},
    {	"Preset 2", { 20.f, 90.f, 0.f, 1.f, 80.f, 90.f, 90.f, 1.f }},
    {	"Preset 3",	{ 30.f, 80.f, 1.f, 2.f, 70.f, 80.f, 80.f, 2.f }},
    {	"Preset 4",	{ 40.f, 70.f, 1.f, 3.f, 60.f, 70.f, 70.f, 3.f }},
    {	"Preset 5",	{ 50.f, 60.f, 1.f, 4.f, 50.f, 60.f, 60.f, 4.f }},
    {	"Preset 6",	{ 60.f, 50.f, 1.f, 5.f, 40.f, 50.f, 50.f, 5.f }},
    {	"Preset 7", { 70.f, 40.f, 1.f, 6.f, 30.f, 40.f, 40.f, 6.f }},
    {	"Preset 8",	{ 80.f, 30.f, 1.f, 7.f, 20.f, 30.f, 30.f, 7.f }},
    {	"Preset 9", { 90.f, 20.f, 1.f, 8.f, 10.f, 20.f, 20.f, 0.f }},
    {	"Preset 10",{ 100.f, 10.f, 1.f, 0.f, 0.f, 10.f, 10.f, 1.f }}
        
    };
    
    for ( int i = 0; i < kNumPresets; i++ )
    {			
        memcpy ( &Presets[ i ], &asEZ_preset[ i ], sizeof ( EZ_preset ));
        
        AU_Presets[ i ].presetNumber = i + 1; // begin at No. 1 for Logic...
        AU_Presets[ i ].presetName = CFStringCreateWithCString(NULL, asEZ_preset[ i ].name, kCFStringEncodingISOLatin1 );
        
    }
    
    
    currentPreset = 0;
    
    strcpy ( editPreset.name, "default" );
    
    for ( int i = 0; i<kNumParameters; i++ )
        editPreset.values[ i ] = 0.f;
    
    // Makes the parameter list smaller.
    enum {
        kGeneric 		= kAudioUnitParameterUnit_Generic,
        kIndexed 		= kAudioUnitParameterUnit_Indexed,
        kBoolean 		= kAudioUnitParameterUnit_Boolean,
        kPercent 		=	kAudioUnitParameterUnit_Percent,
        kSeconds 		= kAudioUnitParameterUnit_Seconds,
        kFrames 		= kAudioUnitParameterUnit_SampleFrames,	
        kPhase 			= kAudioUnitParameterUnit_Phase,
        kRate 			= kAudioUnitParameterUnit_Rate,
        kHertz 			= kAudioUnitParameterUnit_Hertz,
        kCents 			= kAudioUnitParameterUnit_Cents,
        kSemitones 	= kAudioUnitParameterUnit_RelativeSemiTones,
        kMIDINote 	= kAudioUnitParameterUnit_MIDINoteNumber,
        kMIDICtrl 	= kAudioUnitParameterUnit_MIDIController,
        kDecibels 	= kAudioUnitParameterUnit_Decibels,
        kGain 			= kAudioUnitParameterUnit_LinearGain,
        kDegrees 		= kAudioUnitParameterUnit_Degrees,
        kMixerFader = kAudioUnitParameterUnit_MixerFaderCurve1,
        kPan 				= kAudioUnitParameterUnit_Pan,	
        kMeters 		= kAudioUnitParameterUnit_Meters,
        kAbsCents 	= kAudioUnitParameterUnit_AbsoluteCents
    };
    
    UInt32 rw_flag = kAudioUnitParameterFlag_IsReadable | kAudioUnitParameterFlag_IsWritable | kAudioUnitParameterFlag_HasCFNameString;
    
#define NAME(X) X, 0, 0,  CFSTR(X)
    static const AudioUnitParameterInfo asParInfo[]= {
        // Name                 Unit		Min             Max             Default		Flags
    {	NAME("Knob_1"),         kGeneric,	0.0f,		100.0f,		50.0f,		rw_flag	},
    {	NAME("Knob_2"),         kGeneric,	0.0f,		127.0f,		0.0f,		rw_flag	},
    {	NAME("Buttons_1"),	kIndexed,	1.0f,		2.0f,		1.0f,		rw_flag	},
    {	NAME("Buttons_2"),	kIndexed,	0.0f,		8.0f,		0.0f,		rw_flag	},
    {	NAME("Slider_1"),	kGeneric,	0.0f,		100.0f,		0.0f,		rw_flag	},
    {	NAME("Slider_2"),	kGeneric,	0.0f,		100.0f,		0.0f,		rw_flag	},
    {	NAME("Slider_3"),	kGeneric,	0.0f,		100.0f,		50.0f,		rw_flag	},
    {	NAME("Display_1"),	kIndexed,	0.0f,		7.0f,		0.0f,		rw_flag	},
        // -----------------------------------------------------
    {   NAME("<noname>"), 0, 0.0f, 0.0f, 0.0f, 0 }
    };
    
    m_asParInfo=asParInfo;
    m_arPar = editPreset.values;
      
    // Fill parameter value latches with default values.
    int iNum=0;
    while (m_asParInfo[iNum].minValue!=0.0f || m_asParInfo[iNum].maxValue!=0.0f) ++iNum;
    
    if (iNum>0)
    {
        for (int i=0; i<iNum; ++i) m_arPar[i]=m_asParInfo[i].defaultValue;
    }
        
    clearBuffers();
    
    mCurrentLearning = -1;
    for (int i = 0; i < 128; ++i)
        mMIDICC[i] = -1;
    // Restore MIDI Learn data from pref file
    CFArrayRef array = (CFArrayRef)CFPreferencesCopyAppValue(DEMO_MIDILEARN, DEMO_PREFS); 
    if (array) {
        for (int i = 0; i < 128; ++i) {
            CFNumberRef n = CFArrayGetValueAtIndex(array, i);
            CFNumberGetValue(n, kCFNumberShortType, &mMIDICC[i]);
        }
        CFRelease(array);
    }
}

CAUGuiDemoAU::~CAUGuiDemoAU(void) 
{
    // Save MIDI Learn data to pref file
    CFArrayRef array = CFArrayCreateMutable(0, 128, 0);
    for (int i = 0; i < 128; ++i) {
        CFNumberRef n = CFNumberCreate(0, kCFNumberShortType, &mMIDICC[i]);
        CFArraySetValueAtIndex(array, i, n);
    }
    CFPreferencesSetAppValue(DEMO_MIDILEARN, array, DEMO_PREFS);
    CFRelease(array);
    int res = CFPreferencesAppSynchronize(DEMO_PREFS);
}

ComponentResult CAUGuiDemoAU::Initialize(void)
{
    return AUEffectBase::Initialize();
}

UInt32 count = 0;

ComponentResult CAUGuiDemoAU::Reset(AudioUnitScope, AudioUnitElement)
{
    count = 0;
   
    return noErr;
}

void CAUGuiDemoAU::clearBuffers()
{
    
}

ComponentResult CAUGuiDemoAU::GetParameterList(AudioUnitScope iScope, AudioUnitParameterID* asIDList, UInt32& riNum)
{
    // Returns the parameter list.
    if (iScope==kAudioUnitScope_Global)
    {
        riNum=0;
        for (riNum=0; m_asParInfo[riNum].minValue!=0.0f || m_asParInfo[riNum].maxValue!=0.0f; ++riNum)
        {
            if (asIDList) asIDList[riNum]=riNum;
        }
        return noErr;
    }
    
    return AUEffectBase::GetParameterList(iScope, asIDList, riNum);
}

ComponentResult CAUGuiDemoAU::GetParameterInfo(AudioUnitScope iScope, AudioUnitParameterID iParID, AudioUnitParameterInfo& rsInfo)
{
    // Returns info on a single parameter.
    if (iScope==kAudioUnitScope_Global)
    {
        memcpy(&rsInfo, &m_asParInfo[iParID], sizeof(AudioUnitParameterInfo));
        return noErr;
    }
    
    return AUEffectBase::GetParameterInfo(iScope, iParID, rsInfo);
}

ComponentResult CAUGuiDemoAU::SaveState( CFPropertyListRef * outData)
{
    ComponentResult result = AUBase::SaveState(outData);
    if (result != noErr)
        return result;
    
    CFMutableDictionaryRef dict = (CFMutableDictionaryRef) *outData;
    CFMutableDataRef cfdata = CFDataCreateMutable(NULL, NULL);
    UInt8 * plaindata = (UInt8*) &editPreset;
    unsigned long plaindatasize = sizeof(EZ_preset);
    
    CFDataAppendBytes(cfdata, plaindata, plaindatasize);
    CFDictionarySetValue(dict, CFSTR("CAUGuiDemo-DATA-KEY"), cfdata);
    CFRelease(cfdata);
    *outData = dict;
    
    return noErr;
}


ComponentResult CAUGuiDemoAU::RestoreState( CFPropertyListRef	inData)
{
    
    ComponentResult result = AUBase::RestoreState(inData);
    if (result != noErr)
        return result;
    
    CFDataRef cfdata = reinterpret_cast<CFDataRef>(CFDictionaryGetValue((CFDictionaryRef)inData, CFSTR("CAUGuiDemo-DATA-KEY")));
    if (cfdata == NULL)
        return kAudioUnitErr_InvalidPropertyValue;
    
    const UInt8 * plaindata = CFDataGetBytePtr(cfdata);
    unsigned long plaindatasize = CFDataGetLength(cfdata);
    
    if ( plaindatasize != sizeof(EZ_preset) ) return kAudioUnitErr_InvalidPropertyValue;
    
    memcpy ( &editPreset, plaindata, sizeof ( EZ_preset ));
    
    // Maybe load samples etc. from additional preset/settings parameters
    
    
    // make sure settings DO change Parameters even when editor is closed
    for ( int i = 0; i < kNumAUParameters; i++ )
    {
        SetParameter( i, kAudioUnitScope_Global, 0, editPreset.values[ i ], 0 );
    }
    
    return noErr;
}

ComponentResult CAUGuiDemoAU::GetParameterValueStrings(AudioUnitScope inScope, AudioUnitParameterID inID, CFArrayRef* tArrayRef)
{
    if (inScope==kAudioUnitScope_Global)
    {
        switch ( inID )
        {
            case kParam_Display_1:
                static const void* ppVal[] =  {
                    CFSTR("Hi,"), 
                    CFSTR("this"),
                    CFSTR("is an"), 
                    CFSTR("indexed"), 
                    CFSTR("parameter"),
                    CFSTR("with"), 
                    CFSTR("Strings"),
                    CFSTR("attached")
                };
                if (tArrayRef) *tArrayRef=CFArrayCreate(NULL, ppVal, 8, NULL);
                    return noErr;
		
        }
	
    }
    return AUBase::GetParameterValueStrings(inScope, inID, tArrayRef);
}

ComponentResult  CAUGuiDemoAU::GetPresets ( CFArrayRef * outData) const
{
    if (!outData) return noErr;
    
    CFMutableArrayRef outArray = CFArrayCreateMutable(NULL, kNumPresets, NULL);
    
    for (long i=0; i < kNumPresets; i++)
    {
        CFArrayAppendValue(outArray, &(AU_Presets[i]));
    }
    
    *outData = (CFArrayRef)outArray;
    
    return noErr;
    
}

OSStatus CAUGuiDemoAU::NewFactoryPresetSet (const AUPreset & inNewFactoryPreset)
{
    long newNumber = inNewFactoryPreset.presetNumber;
    
    if ( newNumber >= kNumPresets )
        return kAudioUnitErr_InvalidPropertyValue;
    
    AU_Presets[ newNumber ].presetName = inNewFactoryPreset.presetName;
    memcpy ( &editPreset, &Presets[ newNumber ], sizeof ( EZ_preset ));
    
    // Maybe load audiofiles etc.
    
    // make sure presets DO change Parameters even when editor is closed    
    for ( int i = 0; i < kNumAUParameters; i++ )
    {
        SetParameter( i, kAudioUnitScope_Global, 0, editPreset.values[ i ], 0 );
    }
    
    return noErr;
}

UInt32 CAUGuiDemoAU::SupportedNumChannels(const AUChannelInfo** pasInfo)
{
    // Returns the list of the i/o configurations.
    long lNum=0;
    for (; m_asChannelInfo && (m_asChannelInfo[lNum].inChannels!=0 || m_asChannelInfo[lNum].outChannels!=0); ++lNum) {};
    if (pasInfo) *pasInfo=m_asChannelInfo;
    return lNum;
}

ComponentResult CAUGuiDemoAU::GetParameter(AudioUnitParameterID iID, AudioUnitScope iScope, AudioUnitElement iElem, Float32& rValue)
{
    // Get the current value of a parameter.
    if (iScope==kAudioUnitScope_Global && iElem==0)
    {
        rValue=m_arPar[iID];
        return noErr;
    }
    
    return AUBase::GetParameter(iID, iScope, iElem, rValue);
}

ComponentResult CAUGuiDemoAU::SetParameter(AudioUnitParameterID iID, AudioUnitScope iScope, AudioUnitElement iElem, Float32 rValue, UInt32 iSchedule)
{
    printf("%d = %f\n", iID, rValue);
    // Set a new value for a parameter.
    if (iScope==kAudioUnitScope_Global && iElem==0)
    {
        m_arPar[iID]=rValue;
        
        // special cases:
        switch ( iID )
        {
            default:
                break;
        }
    }
    
    return AUBase::SetParameter(iID, iScope, iElem, rValue, iSchedule);
}

// -----------------------------------------------------------------------------
void	CAUGuiDemoAU::HandleControlChange(	int 	inChannel,
                                                UInt8 	inController,
                                                UInt8 	inValue,
                                                long	inStartFrame)
{
    int id = -1;
    if (mCurrentLearning != -1) {
        mMIDICC[inController] = mCurrentLearning;
        mCurrentLearning = -1;
    }
    id = mMIDICC[inController];
    if (id > -1) {
        struct AudioUnitParameter param = {
            GetComponentInstance(),
            id,
            kAudioUnitScope_Global,
            0
        };
        float val = inValue/127.;
        float min = m_asParInfo[id].minValue;
        float max = m_asParInfo[id].maxValue;
        if (m_asParInfo[id].unit != kAudioUnitParameterUnit_Indexed) {
            AUParameterSet(0, 0, &param, min+(max-min)*val, 0);
        } else {
            AUParameterSet(0, 0, &param, int(min+(max-min+.5)*val), 0);
        }
    }
}

ComponentResult	CAUGuiDemoAU::GetPropertyInfo( AudioUnitPropertyID inID, AudioUnitScope inScope,
                                               AudioUnitElement inElement, UInt32 & outDataSize,
                                               Boolean & outWritable)
{
    
    switch ( inID )
    {
        case kInternalParameterProperyID:
            // Internal parameters being read out during timed idle() calls
            outDataSize = kNumInternalParameters * sizeof(float);
            outWritable = false;
            return noErr;
            
        case kProperty_MIDILearn:
            // Set with param number for MIDI learn
            outDataSize = sizeof(int);
            outWritable = true;
            return noErr;
            
        default:
            return AUMIDIEffectBase::GetPropertyInfo(inID, inScope, inElement, outDataSize, outWritable);            
    }
}

ComponentResult	CAUGuiDemoAU::GetProperty( AudioUnitPropertyID inID,
                                           AudioUnitScope inScope,
                                           AudioUnitElement inElement,
                                           void * outData)
{
    switch ( inID )
    {
        case kInternalParameterProperyID:
            // Internal parameters being read out during timed idle() calls
            *((float **)(outData)) = internalParameters;
            return noErr;
            
        case kProperty_MIDILearn:
            // Set with param number for MIDI learn
            *((int *)(outData)) = mCurrentLearning;
            return noErr;
            
        default:
            return AUMIDIEffectBase::GetProperty(inID, inScope, inElement, outData);
    }
}

// -----------------------------------------------------------------------------
ComponentResult	CAUGuiDemoAU::SetProperty( AudioUnitPropertyID inID,
                                    AudioUnitScope inScope,
                                    AudioUnitElement inElement,
                                    const void *inData,
                                    UInt32 inDataSize)
{
    switch ( inID )
    {
        case kProperty_MIDILearn:
            // Set with param number for MIDI learn
            if (mCurrentLearning == *((int *)(inData))) {
                // Clear MIDI control for this parameter
                for (int i = 0; i < 128; ++i)
                    if (mMIDICC[i] == mCurrentLearning)
                        mMIDICC[i] = -1;
                mCurrentLearning = -1;
            } else {
                mCurrentLearning = *((int *)(inData));
            }
            return noErr;
            
        default:
            return AUMIDIEffectBase::SetProperty(inID, inScope, inElement, inData, inDataSize);
    }
}

int CAUGuiDemoAU::GetNumCustomUIComponents () 
{
    return 1;
}

void CAUGuiDemoAU::GetUIComponentDescs (ComponentDescription* inDescArray)
{
    
    inDescArray->componentType = kAudioUnitCarbonViewComponentType;
    inDescArray->componentSubType = 'CAdV';
    inDescArray->componentManufacturer = 'ACME';
    inDescArray->componentFlags = 0;
    inDescArray->componentFlagsMask = 0;
}

OSStatus CAUGuiDemoAU::ProcessBufferLists(AudioUnitRenderActionFlags& iFlags, const AudioBufferList& sInBufs,  AudioBufferList& sOutBufs, UInt32 iFrames)
{
    UInt32 numChannels = sInBufs.mNumberBuffers;
    UInt32 numOutChannels = sOutBufs.mNumberBuffers;
    
    float rms[ 2 ];
    rms[ 0 ] = rms[ 1 ] = 0.f;
    
    if ( numOutChannels < numChannels ) numChannels = numOutChannels;
    
    if ( numChannels < 0 ) numChannels = 0;
    if ( numChannels > 2 ) numChannels = 2;
        
    /*************************************************
        beatAndTempoProc
        
        this stuff tells you where you are in the
        song. - It's not used here, but it's cool
	*************************************************/
    Float64 outCurrentBeat; 
    Float64 outCurrentTempo;
    
    if ( mHostCallbackInfo.beatAndTempoProc != NULL )
    {
        mHostCallbackInfo.beatAndTempoProc ( mHostCallbackInfo.hostUserData,
                                             &outCurrentBeat,
                                             &outCurrentTempo );
    }
    
    for ( int channel = 0; channel < numChannels; channel++ )
    {
	
        float* input	=(float*)(sInBufs.mBuffers[channel].mData);
        float* output	=(float*)(sOutBufs.mBuffers[channel].mData);
        
        long sampleFrames = iFrames;
	
        float sample;
	
        for ( int i = 0; i < sampleFrames; i++ )
        {
            sample = (*input++);
            rms[ channel ] += sample * sample;
            (*output++) = sample;
        }
    }
    
    internalParameters[ kRMS_Left ] = sqrt( rms[ 0 ] / (float) iFrames );
    internalParameters[ kRMS_Right ] = sqrt( rms[ 1 ] / (float) iFrames );
    
    return noErr;
}

