/*	Copyright: 	 Copyright 2002 Urs Heckmann. All rights reserved.

	CAUGuiDemoAU.h
	
=============================================================================*/

#ifndef __CAUGuiDemoAU_h__
#define __CAUGuiDemoAU_h__

#ifndef PI
#define PI (3.1415926535897932384626433832795)
#endif

#define kProperty_MIDILearn 100000

#include <AudioUnit/AudioUnitCarbonView.h>
#include "AUMIDIEffectBase.h"

#include "CAUGuiDemoCsts.h"
#include "CAUGuiConfig.h"

//#include "EZ_AU_preset.h"

enum
{
	kParam_Knob_1 = 0,
	kParam_Knob_2,
	kParam_Buttons_1,
	kParam_Buttons_2,
	kParam_Slider_1,
	kParam_Slider_2,
	kParam_Slider_3,
	kParam_Display_1,
	
	// You can use non-standard parameters after the standard ones
	
	kNumParameters,
	
	
	kNumAUParameters = kNumParameters,// here it's the same number...
	
	kNumPresets = 10
	
};

typedef struct EZ_preset
{
	char			name[ 32 ];
	float			values[ kNumParameters ];
	
	// put whatever you like here
	// i.e. Audiofile References
	
	// Don't use pointers here, just plain data!!!

} EZ_preset;

class CAUGuiDemoAU : public AUMIDIEffectBase
{

	public:
		
		CAUGuiDemoAU(AudioUnit);
		
		~CAUGuiDemoAU();
		
		ComponentResult		Initialize(void);
	
		ComponentResult		Reset	(AudioUnitScope, AudioUnitElement);
	
		ComponentResult		GetParameterList (AudioUnitScope, AudioUnitParameterID*, UInt32&);
	
		UInt32				SupportedNumChannels (const AUChannelInfo**);
		
		ComponentResult	GetParameterInfo( 	AudioUnitScope inScope,
											AudioUnitParameterID inParameterID,
											AudioUnitParameterInfo &outParameterInfo );
		
		ComponentResult		SaveState( CFPropertyListRef * outData);
		
		ComponentResult		RestoreState( CFPropertyListRef	inData);
		
		ComponentResult GetParameterValueStrings(	AudioUnitScope inScope, 
													AudioUnitParameterID inID,
													CFArrayRef* tArrayRef);

		
		ComponentResult		GetPresets ( CFArrayRef * outData) const;

		OSStatus NewFactoryPresetSet (const AUPreset & inNewFactoryPreset);


		ComponentResult	GetPropertyInfo( AudioUnitPropertyID inID, AudioUnitScope inScope,
                                                AudioUnitElement inElement, UInt32 & outDataSize,
                                                Boolean & outWritable);
												
		ComponentResult	GetProperty( AudioUnitPropertyID inID,
                                            AudioUnitScope inScope,
                                            AudioUnitElement inElement,
                                            void * outData);
											
                ComponentResult	SetProperty( AudioUnitPropertyID inID,
                                                           AudioUnitScope inScope,
                                                           AudioUnitElement inElement,
                                                           const void *inData,
                                             UInt32 inDataSize);
                void	HandleControlChange(	int 	inChannel,
                                                                UInt8 	inController,
                                                                UInt8 	inValue,
                                                                long	inStartFrame);

                    // NEW in Dec 02 Dev Tools
                int	GetNumCustomUIComponents ();
		void 	GetUIComponentDescs (ComponentDescription* inDescArray);


		ComponentResult 	GetParameter (AudioUnitParameterID, AudioUnitScope,	AudioUnitElement, Float32&);

		ComponentResult 	SetParameter (AudioUnitParameterID,	AudioUnitScope,	AudioUnitElement, Float32, UInt32);

		OSStatus			ProcessBufferLists (AudioUnitRenderActionFlags&, const AudioBufferList&, AudioBufferList&, UInt32);
                
                ComponentResult	Version()
                {
                    return DEMO_VERSION;
                }
                

private:
	
		
		void				clearBuffers(); // Unused :-)
	

		// Variables for plugin maintainance
		
		const AudioUnitParameterInfo*	m_asParInfo;		// array of parameter infos
		const AUChannelInfo*			m_asChannelInfo;	// array of i/o configurations
		float*							m_arPar;			// parameter value latches
		
		// Internal Parameters to be exchanged between AU and View but are not common parameters
		
		// These are read out via timer, suitable for VUmeters...
		
		float							internalParameters[ kNumInternalParameters ];
		
		// The "dirty" Parameters when editing a Preset
		
		EZ_preset						editPreset;
		
		// The default presets
		
		EZ_preset						Presets[ kNumPresets ];
		
		// The Apple-way Prest name + number
		
		AUPreset						AU_Presets[ kNumPresets ];
		
		// The Preset that's currently edited
		
		int								currentPreset;

                // For MIDI Learn
                short 			mMIDICC[128];   // mMIDICC[CC] is the param to change when receiving a CC. -1 is not bound
                short 			mCurrentLearning; // The parameter ID for the current MIDI learning. (-1 is none)        
};

#endif // __CAUGuiDemoAU_h__

