//
//  AUGUIController.m
//  auguiCocoa
//
//  Created by Airy Andr on Sat May 29 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "AUGUIView.h"
#import "AUGUIController.h"
#import "AUGUIParameterBinding.h"


static void ParameterListenerDispatcher (void *inRefCon, void *inObject, const AudioUnitParameter *inParameter, Float32 inValue) {
    AUGUIController *SELF = (AUGUIController *)inRefCon;
    [SELF _parameterListener:inObject parameter:inParameter value:inValue];
}

@implementation AUGUIController
+ (void)initialize {
    if (self == [AUGUIController class]) {
        [self exposeBinding:@"bindings"];
    }
}

#pragma mark ____ (INIT /) DEALLOC ____
- (id)init {
    [super init];
    mBindings = [[NSMutableArray arrayWithCapacity:10] retain];
    return self;
}

- (void)dealloc {
    [self _removeListeners];
    [mBindings release];
}

- (id)copyWithZone:(NSZone *)zone {
    AUGUIController *newCopy = [[AUGUIController alloc] init];
    [newCopy setBindings:mBindings];
    return newCopy;
}


#pragma mark ____ PUBLIC FUNCTIONS ____

- (AudioUnit)AU {
    return [_cocoaView AU];
}

- (AUGUIView *)cocoaView {
    return _cocoaView;
}

- (NSMutableArray *) bindings
{
    return mBindings;
}

- (void) setBindings: (NSArray *)newBindings
{
    if (mBindings != newBindings)
    {
        [mBindings release];
        mBindings = [[NSMutableArray alloc] initWithArray: newBindings];
    }
}

- (AUGUIParameterBinding *) addBinding:(NSControl *)control withProperty:(NSString *)string forID:(int)paramID;
{
    AUGUIParameterBinding *binding = [[AUGUIParameterBinding alloc] init];
    [binding setControl:control];
    [binding setProperty:string];
    [binding setParamID:paramID];
    [self registerBinding:binding];
    return binding;
}

- (void)registerBinding:(AUGUIParameterBinding *)binding {
    if (!mBindings)
        mBindings = [[NSMutableArray arrayWithCapacity:10] retain];
    [mBindings addObject:binding];
}

- (void)registerParameter:(int)paramID forControl:(NSControl *)control; {
    AudioUnitParameter param =  { [_cocoaView AU], paramID, kAudioUnitScope_Global, 0 };
    AUListenerAddParameter (mParameterListener, 0, &param);    
}

- (void)setParameter:(int)paramID fromControl:(NSControl *)control {
    AudioUnitParameter param =  { [_cocoaView AU], paramID, kAudioUnitScope_Global, 0 };
    AUParameterSet( mParameterListener, control, &param, [control floatValue], 0);
}

#pragma mark ____ PRIVATE FUNCTIONS ____
- (void)_addListeners {
    NSAssert (	AUListenerCreate(	ParameterListenerDispatcher, self, 
                                        CFRunLoopGetCurrent(), kCFRunLoopDefaultMode, 0.100, // 100 ms
                                        &mParameterListener	) == noErr,
                @"[AUGUIView _addListeners] AUListenerCreate()");
    int i;
    for (i = 0; i < [mBindings count]; ++i) {
        AUGUIParameterBinding *binding = [mBindings objectAtIndex:i];
        AudioUnitParameter param =  { [_cocoaView AU], [binding paramID], kAudioUnitScope_Global, 0 };
        int err = AUListenerAddParameter (mParameterListener,
                                          [binding control],
                                          &param);    
        err = AUParameterSet( mParameterListener, [binding control], &param, [[binding control] floatValue], 0);
    }
}

- (void)_removeListeners {
}

- (void)_synchronizeUIWithParameterValues {
}

- (void)_parameterListener:(void *)inObject parameter:(const AudioUnitParameter *)inParameter value:(Float32)inValue {
    NSControl *control = (NSControl *)inObject;
    [control setFloatValue:inValue];
}
@end
