//
//  AUGUIKnob.m
//  auguiCocoa
//
//  Created by Airy Andr on Fri May 21 2004.
//  Copyright (c) 2004 Airy. All rights reserved.
//

#import "AUGUIKnob.h"

@implementation AUGUIKnobCell 

// ---------------------------------------------------------
//  Initialization
// ---------------------------------------------------------
- (void) normalizeValue{
    if (_minValue >= _maxValue)
        _maxValue = _minValue + 0.00001;
    if (_minValue > _value)
        _value = _minValue;
    if (_maxValue < _value)
        _value = _maxValue;
}

- (void) defaultValues {
    [self setKnobImage:0];
    [self setBackgroundImage:0];
    
    [self setKnobName:@"defaultKnob"];
    [self setBackgroundName:@"defaultKnobBack"];
    
    [self setMinValue:0.];
    [self setMaxValue:1.];
    [self setMinAngle:-135.];
    [self setMaxAngle:135.];
    
    [self setXOffset:0.];
    [self setYOffset:0.];
    
    [self setDoubleValue:0.];
    
    [self setIntegralValues:FALSE];
}


- (id)init {
    self = [super init];
    if (self) {
        [self defaultValues];
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)coder {
    self = [super initWithCoder:coder];
    [self defaultValues];
    if ([coder allowsKeyedCoding]) {
	[self setBackgroundName:[coder decodeObjectForKey: @"background"]];
	[self setKnobName:[coder decodeObjectForKey: @"knob"]];
	_minValue = [coder decodeDoubleForKey: @"minValue"];
	_maxValue = [coder decodeDoubleForKey: @"maxValue"];
	_value = [coder decodeDoubleForKey: @"value"];
	_minAngle = [coder decodeDoubleForKey: @"minAngle"];
	_maxAngle = [coder decodeDoubleForKey: @"maxAngle"];
	_xOffset = [coder decodeDoubleForKey: @"xOffset"];
	_yOffset = [coder decodeDoubleForKey: @"yOffset"];
    } else {
	[self setBackgroundName:[coder decodeObject]];
	[self setKnobName:[coder decodeObject]];
        [coder decodeValueOfObjCType:@encode(double) at:&_minValue];
        [coder decodeValueOfObjCType:@encode(double) at:&_maxValue];
        [coder decodeValueOfObjCType:@encode(double) at:&_value];
        [coder decodeValueOfObjCType:@encode(double) at:&_minAngle];
        [coder decodeValueOfObjCType:@encode(double) at:&_maxAngle];
        [coder decodeValueOfObjCType:@encode(double) at:&_xOffset];
        [coder decodeValueOfObjCType:@encode(double) at:&_yOffset];
    }
    [self normalizeValue];
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder {
    [super encodeWithCoder:coder];
    if ([coder allowsKeyedCoding]) {
	[coder encodeObject: _backgroundName forKey: @"background"];
	[coder encodeObject: _knobName forKey: @"knob"];
	[coder encodeDouble: _minValue forKey: @"minValue"];
	[coder encodeDouble: _maxValue forKey: @"maxValue"];
	[coder encodeDouble: _value forKey: @"value"];
	[coder encodeDouble: _minAngle forKey: @"minAngle"];
	[coder encodeDouble: _maxAngle forKey: @"maxAngle"];
	[coder encodeDouble: _xOffset forKey: @"xOffset"];
	[coder encodeDouble: _yOffset forKey: @"yOffset"];
	[coder encodeBool: _integralValues forKey: @"integral"];
    } else {
	[coder encodeObject: _backgroundName];
	[coder encodeObject: _knobName];
	[coder encodeDouble: _minValue];
	[coder encodeDouble: _maxValue];
	[coder encodeDouble: _value];
	[coder encodeDouble: _minAngle];
	[coder encodeDouble: _maxAngle];
	[coder encodeDouble: _xOffset];
	[coder encodeDouble: _yOffset];
	[coder encodeBool: _integralValues];
    }
}

- (void)dealloc {
    [super dealloc];
}

- (void)sendActionToTarget {
    if ([self target] && [self action]) {
        [(NSControl *)[self controlView] sendAction:[self action] to:[self target]];
    }
}

- (void)setBackgroundName:(NSString *)name {
    [name retain];
    [_backgroundName release];
    _backgroundName = name;
    NSImage *image = [NSImage imageNamed:name];
    if (!image && [name isEqualToString:@"defaultKnobBack"]) {
        // A "defaultKnobBack" image is in the augui Framework
        NSBundle* bundle = [NSBundle bundleForClass: [self class]];
        image = [[[NSImage alloc] initByReferencingFile: [bundle pathForImageResource: name]] autorelease];
    }
    [self setBackgroundImage:image];
}

- (void)setKnobName:(NSString *)name {
    [name retain];
    [_knobName release];
    _knobName = name;
    NSImage *image = [NSImage imageNamed:name];
    if (!image && [name isEqualToString:@"defaultKnob"]) {
        // A "defaultKnob" image is in the augui Framework
        NSBundle* bundle = [NSBundle bundleForClass: [self class]];
        image = [[[NSImage alloc] initByReferencingFile: [bundle pathForImageResource: name]] autorelease];
    }
    [self setKnobImage:image];
}

- (void)setBackgroundImage:(NSImage *)image {
    [image retain];
    [_background release];
    _background = image;
}

- (void)setKnobImage:(NSImage *)image {
    [image retain];
    [_knob release];
    _knob = image;
}

- (NSString *)backgroundName {
    return [[_backgroundName retain] autorelease];
}

- (NSString *)knobName {
    return [[_knobName retain] autorelease];
}

- (NSImage *)background {
    return [[_background retain] autorelease];
}

- (NSImage *)knob {
    return [[_knob retain] autorelease];
}

- (double) doubleValue{
    return _value;
}

- (double)minValue{
    return _minValue;
}

- (double)maxValue{
    return _maxValue;
}

- (void)setDoubleValue:(double)val{
    _value = val;
    [self normalizeValue];
    [self sendActionToTarget];
}

- (void)setMinValue:(double)min{
    _minValue = min;
    [self normalizeValue];
}

- (void)setMaxValue:(double)max{
    _maxValue = max;
    [self normalizeValue];
}

- (double)minAngle{
    return _minAngle;
}

- (double)maxAngle{
    return _maxAngle;
}

- (void)setMinAngle:(double)min{
    _minAngle = min;
}

- (void)setMaxAngle:(double)max{
    _maxAngle = max;
}

- (double)xOffset{
    return _xOffset;
}

- (double)yOffset{
    return _yOffset;
}

- (void)setXOffset:(double)offset{
    _xOffset = offset;
}

- (void)setYOffset:(double)offset{
    _yOffset = offset;
}

- (BOOL)integralValues{
    return _integralValues;
}

- (void)setIntegralValues:(BOOL)integral{
    _integralValues = integral;
}

// Tracking mouse stuff
- (BOOL)startTrackingAt:(NSPoint)startPoint inView:(NSView *)controlView
{
    return TRUE;
}

- (BOOL)continueTracking:(NSPoint)lastPoint at:(NSPoint)currentPoint inView:(NSView *)controlView
{
    double deltaX = currentPoint.x - lastPoint.x;
    double deltaY = currentPoint.y - lastPoint.y;
    double value = [self doubleValue];
    value += (deltaX + deltaY)*([self maxValue]-[self minValue])/300.;
    [self setDoubleValue:value];
    [ controlView setNeedsDisplay:TRUE ];
    return true;
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)flag
{
    // We want to continue tracking event when the mouse is outside the view
    return [super trackMouse:theEvent inRect:NSMakeRect(-1e10,-1e10,2e10,2e10) ofView:controlView untilMouseUp:flag];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
    // Drawing background
    if (_background) {
        [_background drawInRect:cellFrame fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
    }
    // Drawing knob
    if (_knob) {
        // Rotating context
        NSAffineTransform* trans = [NSAffineTransform transform];
        [trans translateXBy:[self xOffset] + cellFrame.size.width * 0.5 yBy:[self yOffset]+cellFrame.size.height * 0.5];
        double theRotation = [self minAngle]+(([self doubleValue]-[self minValue])/([self maxValue]-[self minValue]))*([self maxAngle]-[self minAngle]);
        [trans rotateByDegrees:-theRotation];
        [trans translateXBy:-cellFrame.size.width * 0.5 yBy:-cellFrame.size.height * 0.5];
        [trans concat];
        
        [_knob drawInRect:cellFrame fromRect:NSZeroRect    operation:NSCompositeSourceOver fraction:1.0];
    }
}

@end

@implementation AUGUIKnob

+ (void)initialize {
    if (self == [AUGUIKnob class]) {
        [self setCellClass: [AUGUIKnobCell class]];
        [self exposeBinding:@"value"];
        [self exposeBinding:@"minValue"];
        [self exposeBinding:@"maxValue"];
        [self exposeBinding:@"doubleValue"];
    }
}

+ (Class)cellClass {
    return [AUGUIKnobCell class];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
    }
    return self;
}

- (void)setBackgroundName:(NSString *)name {
    [[self cell] setBackgroundName:name];
    [self setNeedsDisplay:TRUE];
}

- (void)setKnobName:(NSString *)name {
    [[self cell] setKnobName:name];
    [self setNeedsDisplay:TRUE];
}

- (void)setBackgroundImage:(NSImage *)image {
    [[self cell] setBackgroundImage:image];
    [self setNeedsDisplay:TRUE];
}

- (void)setKnobImage:(NSImage *)image {
    [[self cell] setKnobImage:image];
    [self setNeedsDisplay:TRUE];
}

- (NSString *)backgroundName {
    return [[self cell] backgroundName];
}

- (NSString *)knobName {
    return [[self cell] knobName];
}

- (NSImage *)background {
    return [[self cell] background];
}

- (NSImage *)knob {
    return [[self cell] knob];
}

- (double)doubleValue{
    return [[self cell] doubleValue];
}

- (float)floatValue{
    return [[self cell] doubleValue];
}

- (double)value{
    return [[self cell] doubleValue];
}

- (double)minValue{
    return [[self cell] minValue];
}

- (double)maxValue{
    return [[self cell] maxValue];
}

- (void)setMinValue:(double)min{
    [[self cell] setMinValue:min];
    [self setNeedsDisplay:TRUE];
}

- (void)setDoubleValue:(double)val{
    printf("Set Double Value to = %lg\n", val);
    [[self cell] setDoubleValue:val];
    [self setNeedsDisplay:TRUE];
}

- (void)setFloatValue:(float)val{
    [[self cell] setDoubleValue:val];
    [self setNeedsDisplay:TRUE];
}

- (void)setValue:(double)val{
    [[self cell] setDoubleValue:val];
    [self setNeedsDisplay:TRUE];
}


- (void)setMaxValue:(double)max{
    [[self cell] setMaxValue:max];
    [self setNeedsDisplay:TRUE];
}

- (double)minAngle{
    return [[self cell] minAngle];
}

- (double)maxAngle{
    return [[self cell] maxAngle];
}

- (void)setMinAngle:(double)min{
    [[self cell] setMinAngle:min];
    [self setNeedsDisplay:TRUE];
}

- (void)setMaxAngle:(double)max{
    [[self cell] setMaxAngle:max];
    [self setNeedsDisplay:TRUE];
}

- (double)xOffset{
    return [[self cell] xOffset];
}

- (double)yOffset{
    return [[self cell] yOffset];
}

- (void)setYOffset:(double)offset{
    [[self cell] setYOffset:offset];
    [self setNeedsDisplay:TRUE];
}

- (void)setXOffset:(double)offset{
    [[self cell] setXOffset:offset];
    [self setNeedsDisplay:TRUE];
}

- (BOOL)integralValues{
    return [[self cell] integralValues];
}

- (void)setIntegralValues:(BOOL)integral{
    [[self cell] setIntegralValues:integral];
    [self setNeedsDisplay:TRUE];
}
@end
